//Saibamaze. Mail Notifier. Used ESP-Mail-Client code repo to read mail.
#include <Servo.h>

#include <SPI.h>
#include <Wire.h>
#include <Adafruit_GFX.h>
#include <Adafruit_SSD1306.h>

#include <Arduino.h>
#if defined(ESP32)
#include <WiFi.h>
#elif defined(ESP8266)
#include <ESP8266WiFi.h>
#else
#endif

#include <ESP_Mail_Client.h>

#define SCREEN_WIDTH 128 // OLED display width, in pixels
#define SCREEN_HEIGHT 64 // OLED display height, in pixels

// Declaration for an SSD1306 display connected to I2C (SDA, SCL pins)
#define OLED_RESET     -1 // Reset pin # (or -1 if sharing Arduino reset pin)
Adafruit_SSD1306 display(SCREEN_WIDTH, SCREEN_HEIGHT, &Wire, OLED_RESET);

//SERVO
Servo servo;

//LED
int ledPin = 13; //Assigned to GPIO Pinout #, not the digital pin #

//Button
int button = 12;
int buttonState = 0;
#define ACTIVATED HIGH

// To use only IMAP functions, you can exclude the SMTP from compilation, see ESP_Mail_FS.h.
//Enter your wifi and password here
#define WIFI_SSID "JBalistreriWireless "
#define WIFI_PASSWORD "1957Balistreri"

//Gmail's imap server host
#define IMAP_HOST "imap.gmail.com"

//Gmails imap port
#define IMAP_PORT 993

/* The log in credentials, use your gmail's app password here */
#define AUTHOR_EMAIL "messagefeederbot@gmail.com"
#define AUTHOR_PASSWORD "yllpoyzvzazwrecj"

/* Print the selected folder update info */
void printPollingStatus(IMAPSession &imap);

/* Callback function to get the Email reading status */
void imapCallback(IMAP_Status status);


/* Print all messages from the message list */
void printMessages(MB_VECTOR<IMAP_MSG_Item> &msgItems, bool headerOnly);

/* Declare the global used IMAPSession object for IMAP transport */
IMAPSession imap;

/* Declare the global used ESP_Mail_Session for user defined session credentials */
ESP_Mail_Session session;

/** Declare the global used IMAP_Config object used for user defined IMAP operating options
 * and contains the IMAP operating result
 */
IMAP_Config config;

void connectWiFi()
{
    //WiFi.mode(WIFI_STA);
    WiFi.disconnect();

    Serial.print("Connecting to AP");

    WiFi.begin(WIFI_SSID, WIFI_PASSWORD);
    while (WiFi.status() != WL_CONNECTED)
    {
        Serial.print(WiFi.status());
        delay(200);
    }

    Serial.println("");
    Serial.println("WiFi connected.");
    Serial.println("IP address: ");
    Serial.println(WiFi.localIP());
    Serial.println();
}

void setup()
{

   servo.attach(2);
   //servo.write(200);
   delay(200);
  
   pinMode(ledPin, OUTPUT);
   pinMode(button, INPUT_PULLUP);
   digitalWrite(button, HIGH);
 
    Serial.begin(115200);

    // SSD1306_SWITCHCAPVCC = generate display voltage from 3.3V internally
  if(!display.begin(SSD1306_SWITCHCAPVCC, 0x3C)) { 
    Serial.println(F("SSD1306 allocation failed"));
    for(;;); // Don't proceed, loop forever
  } delay(30);
  
   display.setTextSize(1);
   display.setTextColor(WHITE);
   display.clearDisplay();

    delay(30);
   #if defined(ARDUINO_ARCH_SAMD)
      while (!Serial)
           ;
    #endif

    Serial.println();

    /*  Set the network reconnection option */
    MailClient.networkReconnect(true);

    connectWiFi();

    /** Enable the debug via Serial port
     * 0 for no debugging
     * 1 for basic level debugging
     *
     * Debug port can be changed via ESP_MAIL_DEFAULT_DEBUG_PORT in ESP_Mail_FS.h
     */
    imap.debug(2);

    /* Set the callback function to get the reading results */
    imap.callback(imapCallback);

    /* Set the session config */
    session.server.host_name = IMAP_HOST;
    session.server.port = IMAP_PORT;
    session.login.email = AUTHOR_EMAIL;
    session.login.password = AUTHOR_PASSWORD;

    session.time.ntp_server = F("pool.ntp.org,time.nist.gov");
    session.time.gmt_offset = 3;
    session.time.day_light_offset = 0;

    /** Assign internet connection handler function
     * in case of lost internet connection for re-listening the mailbox.
     */
    session.network_connection_handler = connectWiFi;

    /* Connect to the server */
    if (!imap.connect(&session /* session credentials */, &config /* operating options and its result */))
        return;


    /* Open or select the mailbox folder to read or search the message */
    if (!imap.selectFolder(F("INBOX")))
        return;
}

void loop()
{
    /* imap.connect and imap.selectFolder or imap.openFolder nedded to be called once prior to listen */

    servo.write(180);//No mail, lower flag
    
    digitalWrite(ledPin, LOW);
    delay(0);

    display.clearDisplay();
    //Print top header
    display.setCursor(27,0);
    display.print("[NO MESSAGES]");
    display.display();
            
    // Listen for mailbox changes
    if (!imap.listen())
        return;

    // Check the changes
    if (imap.folderChanged())
        printPollingStatus(imap);
    // To stop listen, use imap.stopListen(); and to listen again, call imap.listen()
    
}

void printPollingStatus(IMAPSession &imap)
{
    // Declare the selected folder info class to get the info of selected mailbox folder 
    SelectedFolderInfo sFolder = imap.selectedFolder();

    // Show the mailbox info 
    ESP_MAIL_PRINTF("\nMailbox status changed\n----------------------\nTotal Messages: %d\n", sFolder.msgCount());

    if (sFolder.pollingStatus().type == imap_polling_status_type_new_message)
    {

        ESP_MAIL_PRINTF("New message %d, has been addedd, reading message...\n", (int)sFolder.pollingStatus().messageNum);

        // we need to stop polling before do anything
        imap.stopListen();

        // Get the UID of new message and fetch
        config.fetch.uid = imap.getUID(sFolder.pollingStatus().messageNum);
        MailClient.readMail(&imap, false);
    }
    else if (sFolder.pollingStatus().type == imap_polling_status_type_remove_message)
        ESP_MAIL_PRINTF("Message %d, has been removed\n\n", (int)sFolder.pollingStatus().messageNum);
    else if (sFolder.pollingStatus().type == imap_polling_status_type_fetch_message)
        ESP_MAIL_PRINTF("Message %d, has been fetched with the argument %s\n\n", (int)sFolder.pollingStatus().messageNum, sFolder.pollingStatus().argument.c_str());
}

// Callback function to get the Email reading status 
void imapCallback(IMAP_Status status)
{
    // Show the result when reading finished 
    if (status.success())
    {
        // Print the result 
         //Get the message list from the message list data 
        IMAP_MSG_List msgList = imap.data();
        printMessages(msgList.msgItems, imap.headerOnly());

         //Clear all stored data in IMAPSession object 
        imap.empty();
    }
}



void printMessages(MB_VECTOR<IMAP_MSG_Item> &msgItems, bool headerOnly)
{

    for (size_t i = 0; i < msgItems.size(); i++)
    {

        /* Iterate to get each message data through the message item data */
        IMAP_MSG_Item msg = msgItems[i];

        if (strlen(msg.subject)){
            delay(300);
           
            ESP_MAIL_PRINTF("Subject: %s\n", msg.subject); //Print message (emails subject)
            display.clearDisplay();
            //Print top header
            display.setCursor(27,0);
            display.print("[NEW MESSAGE]");
            //Print Message (Subject)
            display.setCursor(0,20);
            display.print(msg.subject);
            display.display();
            Serial.println("Printed message to screen");

            digitalWrite(ledPin, HIGH); //Light up LED
            delay(200);

            while (digitalRead(button) == ACTIVATED){ //Keep message displayed until button is pressed.
              delay(100);
              servo.write(5);
            }

        }
    }
}
